@@ -5,6 +5,7 @@ use dashmap::DashMap;
 use either::Either;
 use once_cell::sync::Lazy;
 use regex::Regex;
+use rustc_hash::{FxHashMap, FxHashSet};
 use serde::{Deserialize, Serialize};
 use std::{
     cell::RefCell,
@@ -182,6 +183,7 @@ impl Options {
         is_module: bool,
         config: Option<Config>,
         comments: Option<&'a SwcComments>,
+        custom_before_pass: impl 'a + swc_ecma_visit::Fold,
     ) -> BuiltConfig<impl 'a + swc_ecma_visit::Fold> {
         let mut config = config.unwrap_or_else(Default::default);
         config.merge(&self.config);
@@ -287,6 +289,7 @@ impl Options {
                 syntax,
                 config.module,
                 comments,
+                custom_before_pass,
             );
 
         let pass = chain!(pass, Optional::new(jest::jest(), transform.hidden.jest));
@@ -826,7 +829,7 @@ impl ModuleConfig {
         scope: RustRc<RefCell<Scope>>,
     ) -> Box<dyn swc_ecma_visit::Fold> {
         let base = match base {
-            FileName::Real(v) => {
+            FileName::Real(v) if !paths.is_empty() => {
                 FileName::Real(v.canonicalize().unwrap_or_else(|_| v.to_path_buf()))
             }
             _ => base.clone(),
@@ -938,12 +941,12 @@ fn default_jsonify_min_cost() -> usize {
 #[serde(deny_unknown_fields, rename_all = "camelCase")]
 pub struct GlobalPassOption {
     #[serde(default)]
-    pub vars: HashMap<String, String>,
+    pub vars: FxHashMap<String, String>,
     #[serde(default = "default_envs")]
-    pub envs: HashSet<String>,
+    pub envs: FxHashSet<String>,
 }
 
-fn default_envs() -> HashSet<String> {
+fn default_envs() -> FxHashSet<String> {
     let mut v = HashSet::default();
     v.insert(String::from("NODE_ENV"));
     v.insert(String::from("SWC_ENV"));
@@ -952,12 +955,14 @@ fn default_envs() -> HashSet<String> {
 
 impl GlobalPassOption {
     pub fn build(self, cm: &SourceMap, handler: &Handler) -> impl 'static + Fold {
+        type ValuesMap = Arc<FxHashMap<JsWord, Expr>>;
+
         fn mk_map(
             cm: &SourceMap,
             handler: &Handler,
             values: impl Iterator<Item = (String, String)>,
             is_env: bool,
-        ) -> HashMap<JsWord, Expr> {
+        ) -> ValuesMap {
             let mut m = HashMap::default();
 
             for (k, v) in values {
@@ -999,23 +1004,49 @@ impl GlobalPassOption {
                 m.insert((*k).into(), expr);
             }
 
-            m
+            Arc::new(m)
         }
 
-        let envs = self.envs;
-        inline_globals(
-            if cfg!(target_arch = "wasm32") {
-                mk_map(cm, handler, vec![].into_iter(), true)
+        let env_map = if cfg!(target_arch = "wasm32") {
+            Arc::new(Default::default())
+        } else {
+            static CACHE: Lazy<DashMap<Vec<String>, ValuesMap>> = Lazy::new(|| Default::default());
+
+            let cache_key = self.envs.iter().cloned().collect::<Vec<_>>();
+            if let Some(v) = CACHE.get(&cache_key).as_deref().cloned() {
+                v
             } else {
-                mk_map(
+                let envs = self.envs;
+                let map = mk_map(
                     cm,
                     handler,
                     env::vars().filter(|(k, _)| envs.contains(&*k)),
                     true,
-                )
-            },
-            mk_map(cm, handler, self.vars.into_iter(), false),
-        )
+                );
+                CACHE.insert(cache_key, map.clone());
+                map
+            }
+        };
+
+        let global_map = {
+            static CACHE: Lazy<DashMap<Vec<(String, String)>, ValuesMap>> =
+                Lazy::new(|| Default::default());
+
+            let cache_key = self
+                .vars
+                .iter()
+                .map(|(k, v)| (k.clone(), v.clone()))
+                .collect::<Vec<_>>();
+            if let Some(v) = CACHE.get(&cache_key) {
+                (*v).clone()
+            } else {
+                let map = mk_map(cm, handler, self.vars.into_iter(), false);
+                CACHE.insert(cache_key, map.clone());
+                map
+            }
+        };
+
+        inline_globals(env_map, global_map)
     }
 }
 
